<?php
/**
* NOTICE OF LICENSE
*
* This source file is subject to the Open Software License (OSL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/osl-3.0.php
*
*  @author    Veebipoed.ee, SEB Payment Gateway
*  @copyright 2015 Veebipoed.ee, SEB Payment Gateway
*  @license   http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
*/

include_once(_PS_MODULE_DIR_.'everypay/lib/CardStorageLibrary.php');

class EveryPayReturnModuleFrontController extends ModuleFrontController
{
    public $ssl = true;
    public $display_column_left = false;
    private $transactionResult;
    private $paymentInfo;
    private $isLiveMode;
    private $isIframe;

    public function init()
    {
        parent::init();
        $this->isLiveMode = Configuration::get("EVERYPAY_LIVE_MODE");
        $this->paymentInfo = $this->getPaymentInfo();
        $this->transactionResult = $this->paymentInfo['payment_state'];
        $this->isIframe = Tools::getValue('content_only');

        if ((Configuration::get($this->module->formatName('create_order')) != 'after') &&
            $this->transactionResult == EveryPay::RETURN_CANCEL
        ) {
            Tools::redirect('index.php?controller=order&step=1');
        } elseif ((Configuration::get($this->module->formatName('create_order')) != 'before') &&
            $this->transactionResult == EveryPay::RETURN_CANCEL
        ) {
            $orderId = $this->getOrderIdByReference($this->paymentInfo['order_reference']);
            Tools::redirect($this->context->link->getPageLink('order', true, NULL, "submitReorder&id_order=${orderId}"));
        }

    }

    public function initContent()
    {
        $order = null;
        $orderConfUrl = null;
        $orderInfoUrl = null;
        $paymentSuccessful = $this->transactionResult === EveryPay::RETURN_SUCCESS
            || $this->transactionResult === EveryPay::RETURN_SETTLED;
        $underPaid = false;
        $overPaid = false;
        $paidAmount = 0;
        $overPaidByAmount = 0;
        if (Configuration::get($this->module->formatName('create_order')) === EveryPay::ORDER_BEFORE ) {
            $orderId = $this->getOrderIdByReference($this->paymentInfo['order_reference']);
            $order = new Order($orderId);
            if ($paymentSuccessful && ($order->getCurrentState() != Configuration::get("PS_OS_PAYMENT"))) {
                $order->setCurrentState(Configuration::get("PS_OS_PAYMENT")); // update status only if not already 'paid'
            }
        } else {
            $orderState = $paymentSuccessful ? Configuration::get('PS_OS_PAYMENT') : Configuration::get('PS_OS_EVERYPAY');
            $cartId = (int) $this->paymentInfo['order_reference'];
            $cart = new Cart($cartId);

            $paidAmount = (float)$this->paymentInfo['initial_amount'];
            $total = (float)$cart->getOrderTotal(true, Cart::BOTH);

            $roundingDecimals = 0;
            $paidAmountRounded = number_format($paidAmount, $roundingDecimals , '.','');
            $totalRounded = number_format($total, $roundingDecimals , '.','');

            # check that cart total was not changed during payment
            $amounts_comparing = bccomp($paidAmountRounded, $totalRounded, $roundingDecimals );
            if (0 > $amounts_comparing) { # if ($paidAmount < $total)
                $underPaid = true;
                $orderState = Configuration::get('PS_OS_ERROR');
                $total = 0; # this custom paid amount does not work with not 'logable' order states
            } elseif (0 < $amounts_comparing) {
                $overPaid = true;
            }

            if (!$cart->orderExists()) {
                $total = (float)$cart->getOrderTotal(true, Cart::BOTH);
                $customer = new Customer($cart->id_customer);
                $this->module->validateOrder(
                    $cart->id,
                    $orderState,
                    $total, # this custom paid amount does not work with not 'logable' order states
                    $this->module->getFormattedPaymentMethod (Tools::getValue('payment_method') ? Tools::getValue('payment_method') :
                        $this->paymentInfo['payment_method'] ? $this->paymentInfo['payment_method'] : 'missing_payment_method'),
                    //$this->module->l('Card payment', 'validation'),

                    null,
                    array(),
                    (int) $this->context->currency->id,
                    false,
                    $customer->secure_key
                );

                if (true == $overPaid) {
                    $overPaidByAmount = $paidAmount - $total;
                }

                if ($this->module->currentOrder) $order = new Order($this->module->currentOrder);
            }
            $orderId = $this->getOrderIdByCartId($cart->id);
            if ($orderId) $order = new Order($orderId);
        }

        # add order payment for underpaid order if no payment exists
        if ($underPaid && $paidAmount > 0) { # these are assigned only when order create after mode is used
            # check if no payment exists:
            $hasPayments = false;
            $orderPayments = $order->getOrderPaymentCollection();
            foreach ($orderPayments as $orderPayment) {
                $hasPayments = true;
                break;
            }

            if (!$hasPayments) {
                $moduleName = $this->module->name;
                $message = "$moduleName: Amount paid ($paidAmount) was less than order total!";
                $this->module->insertOrderMessage($message, $order);

                if (!$order->addOrderPayment($paidAmount)) {
                    PrestaShopLogger::addLog('EveryPayReturnModuleFrontController::initContent - Cannot save Order Payment',
                        3, null, 'Order', (int)$orderId, true);

                    // throw new PrestaShopException('Can\'t save Order Payment'); # possible use
                }
            }
        } elseif ($overPaidByAmount > 0){ # only appears when order is overpaid and is not created yet

            $moduleName = $this->module->name;
            $message = "$moduleName: Amount paid ($paidAmount) was more than order total!";
            $this->module->insertOrderMessage ($message, $order);

            if (!$order->addOrderPayment($overPaidByAmount)) {
                PrestaShopLogger::addLog('EveryPayReturnModuleFrontController::initContent - Cannot save Order Payment', 3, null, 'Order', (int) $orderId, true);

                // throw new PrestaShopException('Can\'t save Order Payment'); # possible use
            }
        }

        if (Validate::isLoadedObject($order) && $order->id_customer == $this->context->customer->id
            && $paymentSuccessful && !$underPaid && !$overPaid) {
            if (isset($this->paymentInfo['cc_details']) && isset($this->paymentInfo['cc_details']['token'])) {
                $this->saveCreditCard($order, $this->paymentInfo['cc_details']);
            }

            //Tools::redirectLink($this->module->getOrderConfUrl($order));
            if ($this->isIframe) {
                $orderConfUrl = $this->module->getOrderConfUrl($order);
            } else {
                Tools::redirectLink($this->module->getOrderConfUrl($order));
            }

        }

        if (Validate::isLoadedObject($order) && $order->id_customer == $this->context->customer->id) {
            $orderInfoUrl = $this->context->link->getPageLink(
                'order-detail',
                true,
                $order->id_lang,
                array(
                    'id_order' => $order->id
                )
            );
        }

        parent::initContent();
        $this->context->smarty->assign(array(
            'transaction_result' => $this->transactionResult,
            'order_reference' => $order ? $order->id : '',
            'order_conf_url' => $orderConfUrl,
            'is_iframe' => $this->isIframe,
            'order_info_url' => $orderInfoUrl,
            'under_paid' => $underPaid,
            'over_paid' => $overPaid
        ));

        $this->setTemplate('return.tpl');
        //$this->setTemplate('module:everypay/views/templates/front/return.tpl'); // PS17 version
    }

    private function saveCreditCard(Order $order, $creditCardInfo) {
        $token = new CardStorageLibrary();
        $token->id_customer = $order->id_customer;
        $token->token = $creditCardInfo['token'];
        $token->card_exp_year = $creditCardInfo['year'];
        $token->card_exp_month = $creditCardInfo['month'];
        $cardType = CardStorageLibrary::$cardTypes[$creditCardInfo['type']];
        $token->card_type = $cardType;
        $token->card_no = $creditCardInfo['last_four_digits'];
        if (!$token->doesExist()) {
            $token->save();
        }
    }

    private function getPaymentInfo() {
        $payment_reference = Tools::getValue("payment_reference");
        $api_username = $this->isLiveMode ? Configuration::get("EVERYPAY_API_USERNAME") : Configuration::get("EVERYPAY_TEST_API_USERNAME");
        $api_secret = $this->isLiveMode ? Configuration::get("EVERYPAY_SHARED_SECRET") : Configuration::get("EVERYPAY_TEST_SHARED_SECRET");

        $response = $this->sendGetRequestWithAuth($this->getPaymentResultUrl($payment_reference, $api_username), $api_username, $api_secret);

        if (EveryPay::DEBUG_LOGGING) {
            $this->module->logError(array('response' => $response), 'ReturnController::getPaymentInfo()');
        }

        return $response;
    }

    // Not used at the moment because Everypay API now sends payment_reference as GET parameter to customer_url
    private function getCurrentPaymentReference() {
        $sql = new DbQuery();
        $sql->select("payment_reference");
        $sql->from("everypay_payments");
        $sql->where("order_reference = " . (int)$this->paymentInfo['order_reference']);
        return Db::getInstance()->executeS($sql)[0]["payment_reference"];
    }

    private function sendGetRequestWithAuth($url, $username, $password) {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
                'Content-Type: application/json'
            )
        );
        curl_setopt($ch, CURLOPT_USERPWD, $username . ":" . $password);
        $response = curl_exec($ch);
        curl_close($ch);
        return json_decode($response, true);
    }

    private function getPaymentResultUrl($payment_reference, $api_username) {
        $baseUrl = $this->isLiveMode ? "https://pay.every-pay.eu/api/v3/payments/" : "https://igw-demo.every-pay.com/api/v3/payments/";//"https://payment.ecommerce.sebgroup.com/api/v3/payments/" : "https://igw-demo.every-pay.com/api/v3/payments/";
        return $baseUrl . $payment_reference . "?api_username=" . $api_username;
    }

    private function getOrderIdBySecureKey($secureKey) {
        $sql = new DbQuery();
        $sql->select("id_order");
        $sql->from("orders");
        $sql->where('secure_key = ' . '"' . $secureKey . '"');
        return Db::getInstance()->getValue($sql);
    }

    private function getOrderIdByReference($orderReference) {
        $sql = new DbQuery();
        $sql->select('*');
        $sql->from('orders');
        $sql->where('reference = ' . '"' . $orderReference . '"');
        return Db::getInstance()->getValue($sql);
    }

    private function getOrderIdByCartId($cartId)
    {
       $sql = new DbQuery();
       $sql->select('id_order')
           ->from('orders')
           ->where('id_cart = ' . (int) $cartId);
       return Db::getInstance()->getValue($sql);
    }

}
